<?php

namespace Drupal\blacksmith\Blacksmith\EntityImporter\FieldFormatter;

use Drupal;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;

/**
 * Class FieldFormatterFactory.
 *
 * @package Drupal\blacksmith\Blacksmith\EntityImporter\FieldFormatter
 */
class FieldFormatterFactory {

  /**
   * Entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * FieldFormatterFactory constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager service.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Defines the correct FieldFormatter for a specific field type.
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $fieldDefinition
   *   The field definition of the field about to be imported.
   *
   * @return \Drupal\blacksmith\Blacksmith\EntityImporter\FieldFormatter\FieldFormatterInterface
   *   The correct FieldFormatted best suited to format to field.
   */
  public function create(FieldDefinitionInterface $fieldDefinition) : FieldFormatterInterface {
    $fieldName = $fieldDefinition->getName();
    $fieldType = $fieldDefinition->getType();

    switch ($fieldType) {
      case 'boolean':
        return new BooleanFieldFormatter($fieldDefinition);

      case 'created':
      case 'changed':
      case 'datetime':
      case 'timestamp':
        return new DateFieldFormatter($fieldDefinition);

        case 'daterange':
        return new DateRangeFieldFormatter($fieldDefinition);

      case 'email':
      case 'list_string':
      case 'string':
      case 'string_long':
      case 'link':
      case 'telephone':
        return new FieldFormatterBase($fieldDefinition);

      case 'text':
      case 'text_long':
        return new FormattedTextFieldFormatter($fieldDefinition);

      case 'text_with_summary':
        return new FormattedTextWithSummaryFieldFormatter($fieldDefinition);

      case 'language':
        return new LanguageFieldFormatter($fieldDefinition);

      case 'integer':
      case 'list_integer':
        return new IntegerFieldFormatter($fieldDefinition);

      case 'float':
      case 'decimal':
      case 'list_float':
        return new FloatFieldFormatter($fieldDefinition);

      case 'file':
        return new FileFieldFormatter($fieldDefinition);

      case 'image':
        return new ImageFieldFormatter($fieldDefinition);

      case 'entity_reference':
        return new EntityReferenceFieldFormatter($fieldDefinition, $this->entityTypeManager);

      case 'entity_reference_revisions':
        return new EntityReferenceRevisionsFieldFormatter($fieldDefinition, $this->entityTypeManager);

      default:
        Drupal::messenger()->addWarning("Missing FieldFormatter for field : $fieldName ($fieldType)");
        return new FieldFormatterBase($fieldDefinition);
    }

  }

}
