<?php

namespace Drupal\blacksmith\Blacksmith\EntityImporter\FieldFormatter;

use Drupal;
use Drupal\Core\File\FileSystemInterface;
use Drupal\blacksmith\Exception\BlacksmithImportSkipField;
use Symfony\Component\DependencyInjection\ContainerAwareTrait;

/**
 * Class ImageFieldFormatter.
 *
 * @package Drupal\blacksmith\Blacksmith\EntityImporter\FieldFormatter
 */
class FileFieldFormatter extends FieldFormatterBase {

  use ContainerAwareTrait;

  /**
   * Drupal token service.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;

  /**
   * Drupal file system service.
   *
   * @var \Drupal\Core\File\FileSystem
   */
  protected $fileSystem;

  /**
   * {@inheritdoc}
   */
  public function __construct($fieldDefinition) {
    parent::__construct($fieldDefinition);
    $this->setContainer();
    $this->token = $this->container->get('token');
    $this->fileSystem = $this->container->get('file_system');
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\blacksmith\Exception\BlacksmithImportSkipField
   */
  protected function formatUniqueValue($value) : array {

    // Assume that a file ID is given if the value is a number.
    if (is_int($value)) {
      return parent::formatUniqueValue($value);
    }

    // Make sure that the value is always provided as an array.
    if (is_string($value)) {
      return $this->formatUniqueValue(['src' => $value]);
    }

    // Load the file from the src and save it in the file directory.
    if (isset($value['src'])) {
      // Fetch the file from the URL.
      // @todo Find a way to get a file that is inside the repository.
      $data = file_get_contents($value['src']);

      if (!$data) {
        throw new BlacksmithImportSkipField('Unable to get the file "' . $value['src'] . '"', NULL);
      }

      // Set the filename.
      if (isset($value['name'])) {
        $filename = trim($value['name']);
      }
      else {
        $fileHeaders = self::parseHeaders($http_response_header);
        $fileType = explode('/', $fileHeaders['Content-Type'])[1];
        // @todo Do not set the name here. The saveData method will do it.
        $filename = 'sample.' . $fileType;
      }

      // Set the file directory and file path.
      $directory = $value['file_directory'] ?? $this->fieldDefinition->getSetting('file_directory');
      $directory = $this->token->replace($directory);
      $fileScheme = Drupal::config('system.file')->get('default_scheme');
      // @todo What the heck is going on???
      $filePath = (empty($directory)) ? "$fileScheme://$filename" : "$fileScheme://$directory/$filename";

      // @todo Find a proper way to name the file.
      /* @var \Drupal\file\Entity\File $file */
      $file = $this->fileSystem->saveData($data, $filePath, FileSystemInterface::EXISTS_RENAME);

      if ($file === FALSE) {
        throw new BlacksmithImportSkipField('Unable to save the file ' . $filePath);
      }

      return [
        'title' => $value['title'] ?? '',
        'target_id' => $file->id(),
      ];
    }

    throw new BlacksmithImportSkipField('You need to specify the image "src"');
  }

  /**
   * Formats the values returned in the header response.
   *
   * @param array $headers
   *   Header response from file_get_contents.
   *
   * @return array
   *   The formatted header response.
   *
   * @see file_get_contents()
   * @see $http_response_header
   */
  public static function parseHeaders(array $headers) : array {
    $head = [];

    foreach ($headers as $key => $value) {
      $t = explode(':', $value, 2);
      if (isset($t[1])) {
        $head[trim($t[0])] = trim($t[1]);
      }
      else {
        $head[] = $value;
        if (preg_match("#HTTP/[\d.]+\s+([\d]+)#", $value, $out)) {
          $head['response_code'] = (int) $out[1];
        }
      }
    }

    return $head;
  }

}
